############################################################################
#	fcn.pl -- program for finding consanguineous loops for each individual and pair-wise connections between each pair of 
#	individuals in extent pedigrees
#
#	F. Liu, Erasmus Medical Center Rotterdam
#	Y. Aulchenko, Erasmus Medical Center Rotterdam, Inst. Cytol. Genet. SD RAS
#
#	This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public 
#	License as published by the Free Software Foundation; either version 1.3 of the License, or (at your option) any later 
#	version.
#     
#	This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY, without even the implied 
#	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#     
#	You should have received a copy of the GNU General Public License along with this program; if not, write to the Free 
#	Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
#	Version 1.3
#	Modified 07-07-2004 -- command line for additional options
#	Modified 17-07-2004 -- descriptive statistics for distributions of loops
#	Modified 08-02-2005 -- compute loop-specific inbreeding/kinship values
#			    -- add the 'inbreeding only' option
#			    -- add the 'help' option
#
#	Contact:
#	f.liu@erasmusmc.nl 
#	or
#	i.aoultchenko@erasmusmc.nl
############################################################################

print "\nfcn version 1.3\n";

if (@ARGV<3){
	print 
"
USAGE: perl fcn.pl OPTION
OPTIONS might be empty, and then the program asks one by one for obligatory options and tries default optional options.
	Options
	-p (pedigree):		the input pedigree file name <obligatory>
	-l (list):		the input list file name <obligatory>
	-so (standard-out): 	the standard output file name <obligatory>
	-co (connection-out): 	the connection output file name <optional>
	-mp (ma/paternal):	the maternal/paternal specific connection output file name <optional>
	-ik (inbr/kin): 	inbreeding/kinship out put file name <optional>
	-io (inbreeding only):	if you want deal with only consanguineous loops (0 or 1) <optional>
	-h (help):		for more information <optional>
For detailed information, see fcn_manual.txt in the folder \"doc\"
";
}


$help = "no";
#specify options in command line
for ($i=0;$i<@ARGV;$i++) {
		
    $line = $ARGV[$i];
    
    $pedigree_file=$1 if ($line =~ /^-p *(.[^\s]*)/);
    $list_file=$1 if ($line =~/^-l *(.[^\s]*)/);
    $standard_output_file=$1 if ($line =~/^-so *(.[^\s]*)/);
    $connection_output_file=$1 if ($line =~/^-co *(.[^\s]*)/);
    $kinship_output_file=$1 if ($line =~/^-ik *(.[^\s]*)/);
    $allele_specific_output_file=$1 if ($line=~/^-mp *(.[^\s]*)/);
    if ($line=~/^-h *(.[^\s]*)/) {$help = $1; print "help is now $help\n";}
    elsif ($line=~/^-help *(.[^\s]*)/){$help = $1}
    elsif ($line=~/^-\? *(.[^\s]*)/){$help = $1};
    $inbronly=$1 if ($line =~/^-io *(.[^\s]*)/);
   

    $pedigree_file=$ARGV[$i+1] if ($line =~ /^-p$/);
    $list_file=$ARGV[$i+1] if ($line =~ /^-l$/);
    $standard_output_file=$ARGV[$i+1] if ($line =~ /^-so$/);
    $connection_output_file=$ARGV[$i+1] if ($line =~ /^-co$/);
    $kinship_output_file=$ARGV[$i+1] if ($line =~ /^-ik$/);
    $allele_specific_output_file=$ARGV[$i+1] if ($line =~ /^-mp$/);
    if ($line =~ /^-h$/) {$help=$ARGV[$i+1]}
    elsif ($line =~ /^-help$/) {$help=$ARGV[$i+1]}
    elsif ($line =~ /^-\?$/) {$help=$ARGV[$i+1]};
    $inbronly=$ARGV[$i+1] if ($line =~ /^-io$/);
    
}


if ($help eq "no"){$help=0}
elsif (!$help){$help=999};

if (!$pedigree_file){interactive_mode(1)};
if (!$list_file){interactive_mode(2)};
if (!$standard_output_file){interactive_mode(3)};




###################
###	MAIN	###
help($help);


$time=localtime();
$time0 = time();
print"Run started at\t$time\n";


# based on options create output file
print	"\nThe following options are in effect:\n",
	"-p: $pedigree_file\n",
	"-l: $list_file\n";
if ($standard_output_file){
	print "-so: $standard_output_file\n";
}if($connection_output_file){
	print "-co: $connection_output_file\n";
}if ($kinship_output_file){
	print "-ik: $kinship_output_file\n";
}if ($allele_specific_output_file){
	print "-mp: $allele_specific_output_file\n";
}if ($inbronly==1){
	print "-io: only consanguineous loops will be analyzed\n";
}print "\n";


#declare universe variables
my @id; my %id; my %idfa; my %idmo; my %idsex; my %idfamily; my @list;
my %id_pathways; my %id_loop_number; my %id_id_pathways; my %id_id_loop_number;
my $inter_file_1="inter_file_1";unlink ($inter_file_1);


my @families=Families($pedigree_file);

foreach $current_family(@families){
	 print STDOUT "analyzing family $current_family...\n";
	 @temp=read_pedigree_data($pedigree_file,$current_family,$list_file);
	 @id= @{$temp[0]},
	 %idfa=%{$temp[1]};
	 %idmo=%{$temp[2]};
	 %idsex=%{$temp[3]};
	 %idfamily=%{$temp[4]};
	 @list=@{$temp[5]};
	 @temp=();

		
	foreach (@list){
		@temp=find_loop($_);
		@{$id_pathways{$_}}=@{$temp[0]};
		$id_loop_number{$_}=$temp[1];
		@temp=();
	}

	if ($inbronly!=1){
	# add a pseudo first degree offspring
	@temp=@list;
	foreach $id1(@list){
		shift(@temp);
		foreach $id2(@temp){
			$add_id=$id1*10000000+$id2;
			push (@id,$add_id);
			push (@add_id,$add_id);			
			$idfa{$add_id}=$id1;
			$idmo{$add_id}=$id2;
		}
	}
	@add_id = sort{$a<=>$b;} keys %{{map{$_,1} @add_id}};
	shift(@add_id) if ($add_id[0]==undef);

	foreach (@add_id) {
		@temp=find_ancestor($_);
		@{$id_pathways{$_}}=@{$temp[0]};
		$id_loop_number{$_}=$temp[1];
		@temp=();
	}
	}
	
	# write to inter file 1
	open (OUT, ">>$inter_file_1") or die "can not open interfile $inter_file_1:$!\n";
	select OUT;
	foreach (@list){
		print "$idfamily{$_},$_,$_,$id_loop_number{$_}",join ",","",@{$id_pathways{$_}};
		print "\n";
	}

	foreach (@add_id) {
		if ($idfamily{$idfa{$_}}){
			print "$idfamily{$idfa{$_}},$idfa{$_},$idmo{$_},$id_loop_number{$_}",join",","",@{$id_pathways{$_}};
			print "\n";
		}
	}
	close (OUT);
	select (STDOUT);
	%id_loop_number=undef;
	%id_pathways=undef;
}

$time=localtime();
$timeN=time();
$took = $timeN-$time0;
print"\nRun stoped at\t$time\nRun time: $took seconds\n";



if ($standard_output_file){
	reformat_s ($inter_file_1,$standard_output_file);
}
if ($connection_output_file){
	reformat_co ($inter_file_1,$connection_output_file);
}
if ($kinship_output_file){
	reformat_k ($inter_file_1,$kinship_output_file);
}
if ($allele_specific_output_file){
	reformat_a ($inter_file_1,$allele_specific_output_file);
}

 unlink ($inter_file_1);
###	END of MAIN	###
###########################



###########################
###	Sub Routines	###

# this subroutine re-format interfile to sdandard outputfile
sub reformat_s {
	my $in=$_[0];
	my $out=$_[1];
	open (OUF,">$out")or die "cannot open output file $out:$!\n";
	select (OUF);
	my ($matrix,$l,$c)=rd ($in);
	my $clmn_length=undef;


	for $a(0..$l){
		if ($matrix->[$a]->[1]==$matrix->[$a]->[2]){
			for $b(4..($#{$matrix->[$a]})){
				if (($b-1)%3==0){
					push 
(@{$clmn_length->[$a]},$matrix->[$a]->[$b+1]+$matrix->[$a]->[$b+2]);
				}
			}
		}else{
			if ($matrix->[$a]->[3]!=0){
				for $b(4..($#{$matrix->[$a]})){
					if (($b+1)%5==0){
						push 
(@{$clmn_length->[$a]}, $matrix->[$a]->[$b+1]+$matrix->[$a]->[$b+3]);
					}

				}
			}
		}
	}

	print 
"family,id1,id2,loop_number,shortest,longest,median,mean,SD\n";
	for $a(0..$l){
		$shortest_length= smallest(\@{$clmn_length->[$a]});
		$longest_length= biggest (\@{$clmn_length->[$a]});
		$mean_length = mean(\@{$clmn_length->[$a]});
		$median_length = median(\@{$clmn_length->[$a]});
		$sd_length = standard_deviation(\@{$clmn_length->[$a]});
		print 
"$matrix->[$a]->[0],",
"$matrix->[$a]->[1],",
"$matrix->[$a]->[2],",
"$matrix->[$a]->[3],",
"$shortest_length,",
"$longest_length,",
"$median_length,",
"$mean_length,",
"$sd_length",
" \n";
	}
	close (OUF);
	select (STDOUT);
	print "\n$standard_output_file is created in the same directory";
}


# this subroutine re-format interfile to connection outputfile
sub reformat_co {
	my $in=$_[0];
	my $out=$_[1];
	open (OUF,">$out")or die "cannot open output file $out:$!\n";
	select (OUF);
	my ($matrix,$l,$c)=rd ($in);
	my $clmn_length=undef;


	for $a(0..$l){
		if ($matrix->[$a]->[1]==$matrix->[$a]->[2]){
			print join ",",@{$matrix->[$a]};
			print "\n";
			for $b(4..($#{$matrix->[$a]})){
				if (($b-1)%3==0){
					push 
(@{$clmn_length->[$a]},$matrix->[$a]->[$b+1]+$matrix->[$a]->[$b+2]);
				}
			}
		}else{
			print join ",",@{$matrix->[$a]}[0..3];
			if ($matrix->[$a]->[3]!=0){
				for $b(4..($#{$matrix->[$a]})){
					if (($b+1)%5==0){
						print 
",$matrix->[$a]->[$b],$matrix->[$a]->[$b+1],$matrix->[$a]->[$b+3]";
						push 
(@{$clmn_length->[$a]}, $matrix->[$a]->[$b+1]+$matrix->[$a]->[$b+3]);
					}

				}
			}
			print "\n";
		}
	}

	close (OUF);
	select (STDOUT);
	print "\n$connection_output_file is created in the same directory\n";
}




# this sub routine reformat interfile to kinship output
sub reformat_k{	
	my $in=$_[0];
	my $out=$_[1];
	my $knshp;my $knshp_st;my $knshp_lg;my$knshp_mn;
	my $inbr;my $inbr_st;my $inbr_lg;my $inbr_mn;
	my $i;
	my $j;
	unlink ($out);

	my ($data2,$l2,$c2)=rd($standard_output_file);
	my ($data,$l,$c)=rd($in);
	
	for $i(0..$l){
		$st=0; $lg=0; $mn=0;
		$st = $data2->[$i+1]->[4];$lg=$data2->[$i+1]->[5];$mn=$data2->[$i+1]->[7];
		if ($data->[$i]->[1]==$data->[$i]->[2]){
			$inbr[$i]=inbrdcf(@{$data->[$i]});
			if ($st!=0 and $st!= 1){$inbr_st[$i]=4*0.5**($st);}else{$inbr_st[$i]=$inbr[$i];}
			if ($lg!=0 and $lg!=1){$inbr_lg[$i]=4*0.5**($lg);}else{$inbr_st[$i]=$inbr[$i];}
			if ($mn!=0 and $mn!=1){$inbr_mn[$i]=4*0.5**($mn);}else{$inbr_mn[$i]=$inbr[$i];}
					}else{
			$knshp[$i]=knshp(@{$data->[$i]});
			if($st!=0 and $st!=1){$knshp_st[$i]=4*0.5**($st+2);}else{$knshp_st[$i]=$knshp[$i]}
			if($lg!=0 and $lg!=1){$knshp_lg[$i]=4*0.5**($lg+2);}else{$knshp_lg[$i]=$knshp[$i]}
			if($mn!=0 and $mn!=1){$knshp_mn[$i]=4*0.5**($mn+2);}else{$knshp_mn[$i]=$knshp[$i]}
		}
	}

	open (OUF,">>$out")or die "cannot open output file $out:$!\n";
	select (OUF);
			printf 
"%8s%8s%8s%12s%12s%12s%12s\n","fid","id1","id2","i/k","i/k_short","i/k_long","i/k_mean";

	for $i(0..$l){
		if ($data->[$i]->[1]==$data->[$i]->[2]){
			printf 
"%8g%8g%8g%12g%12g%12g%12g\n",@{$data->[$i]}[0..2],$inbr[$i],$inbr_st[$i],$inbr_lg[$i],$inbr_mn[$i];
			}else{
			printf 
"%8g%8g%8g%12g%12g%12g%12g\n",@{$data->[$i]}[0..2],$knshp[$i],$knshp_st[$i],$knshp_lg[$i],$knshp_mn[$i];
		}
	}
	close (OUF);
	select (STDOUT);
	print "$kinship_output_file is created in the same directory\n";
}


# this sub routine reformat interfile1 to allele specific pathway output
# paternal and maternal allele specific pathways
sub reformat_a{
	my $in=$_[0];
	my $out=$_[1];
	my $data;
	my $l;
	my $c;
	my $i;
	my $j;
	my @temp;

	($data,$l,$c)=rd($in);
	for $i(0..$l){
		if ($data->[$i]->[1]!=$data->[$i]->[2]){
			for $j(4..$#{$data->[$i]}){
				if (($j+1)%5==0){
					if ($data->[$i]->[$j+2]==1){
						#print "before: $data->[$i]->[$j+2]\t";
						$data->[$i]->[$j+2]="p";
						#print "after: $data->[$i]->[$j+2]\n";
					}elsif ($data->[$i]->[$j+2]==2){
						$data->[$i]->[$j+2]="m";
					}elsif ($data->[$i]->[$j+2]==0){
						$data->[$i]->[$j+2]="x";
					}if ($data->[$i]->[$j+4]==1){
						$data->[$i]->[$j+4]="p";
					}elsif ($data->[$i]->[$j+4]==2){
						$data->[$i]->[$j+4]="m";
					}elsif ($data->[$i]->[$j+4]==0){
						$data->[$i]->[$j+4]="x";
					}	
				}	
			}
		} else {
			push (@temp,@{$data->[$i]}[0..3]);
			for $j(4..$#{$data->[$i]}){
				if ($j%3==1){
					push (@temp,$data->[$i]->[$j],$data->[$i]->[$j+1],"p",$data->[$i]->[$j+2],"m");
				}
			}
			@{$data->[$i]}=@temp;
			@temp=();
		}
	}

	open (OUT,">$out") or die"$!\n";
	select (OUT);
	for $i(0..$l){
		print join ",",@{$data->[$i]};
		print "\n";
	}

	close (OUT);
	select (STDOUT);
	print "$allele_specific_output_file is created in the same directory\n";
}




# this sub routine calculate how many inbr loops for specified id
# and length of each paternal and maternal pathways
# require universe variables: @id,%idfa,%idmo
# return paternal and maternal pathways and total loop number
# format find_loop($interested_id);
sub find_loop{
	my $interested_id=$_[0];
	my $loop_number=0;
	my @pathways;
	###	start form the interested person 
	###	traverse binary tree 
	###	once find the aim, save a way
	foreach $aim(@id){	
		my @total=();
		my $temp=undef;
		my $temp2=undef;
		my $temp1=undef;
		my $temp3=undef;
		my $temp4=undef;
		my $i=0;
		my $j=0;
		my $return=undef;
		my $way=undef;
		@way=();
	
		preorder ($interested_id);
		sub preorder {
			push (@total,$_[0]);

			if ($idfa{$_[0]}!=0) {
				preorder ($idfa{$_[0]});
				preorder ($idmo{$_[0]});
			}
			if ($idfa{$_[0]}==$aim or $idmo{$_[0]}==$aim){
				push (@way,@total);
			}
			pop (@total);
		}

		###	compare ways, find a pair of uniq ways
		$j=0;
		$temp2=$#way;
		for (0..$temp2){
			if ($way[$temp2-$_]!=$interested_id){
				$temp=pop(@way);
				push (@{$way->[$j]},$temp);
			}
			else {pop (@way);$j++;}
		}
		
		$i=0;
		$j=0;
		
		for ($i..$#$way){	
			$j=$i+1;
			for ($j..$#$way){
				$return=if_2_arys_have_same_elements(\@{$way->[$i]},\@{$way->[$j]});
				if ($return==0){
					push (@pathways,$aim,$#{$way->[$i]}+2,$#{$way->[$j]}+2);
					$loop_number++;
				}$j++;
			}$i++;
		}
	}return (\@pathways,$loop_number);
}


# this sub calculate how many common ancestors for a pair of individuals
# and length of paternal and maternal pathways to each common ancestor
# require universe variables: @id,%idfa,%idmo,%idsex
# return paternal and maternal pathways and total loop number
# format (\@pathways,$loop_number)=find_ancestor($interested_id);
sub find_ancestor {
	my $interested_id=$_[0];
	my $loop_number=0;
	my @pathways;
	###	start form the interested person 
	###	traverse binary tree 
	###	once find the aim, save a way
	foreach $aim(@id){	
		my @total=();
		my $temp=undef;
		my $temp2=undef;
		my $temp1=undef;
		my $temp3=undef;
		my $temp4=undef;
		my $i=0;
		my $j=0;
		my $return=undef;
		my $way=undef;
		@way=();
	
		preorder ($interested_id);
		sub preorder {
			push (@total,$_[0]);

			if ($idfa{$_[0]}!=0) {
				preorder ($idfa{$_[0]});
				preorder ($idmo{$_[0]});
			}
			if ($idfa{$_[0]}==$aim or $idmo{$_[0]}==$aim){
				push (@way,@total);
			}
			pop (@total);
		}

		###	compare ways, find a pair of uniq ways
		$j=0;
		$temp2=$#way;
		for (0..$temp2){
			if ($way[$temp2-$_]!=$interested_id){
				$temp=pop(@way);
				push (@{$way->[$j]},$temp);
			}
			else {pop (@way);$j++;}
		}
		
		$i=0;
		$j=0;
		for ($i..$#$way){	
			$j=$i+1;
			for ($j..$#$way){
				$return=if_2_arys_have_same_elements(\@{$way->[$i]},\@{$way->[$j]});
				if ($return==0){
					push (@pathways,$aim,$#{$way->[$i]}+1);
					if ($#{$way->[$i]}>0){
						push(@pathways,$idsex{$way->[$i]->[$#{$way->[$i]}-1]});
					}elsif ($#{$way->[$i]}==0){
						push (@pathways,$idsex{$aim});
						#print "sex:$idsex{$aim}\n"; 			
					}else {
						push (@pathways,0);
					}

					push (@pathways,$#{$way->[$j]}+1);
					if($#{$way->[$j]}>0){
						push (@pathways,$idsex{$way->[$j]->[$#{$way->[$j]}-1]});
					}elsif ($#{$way->[$j]}==0){
						push (@pathways,$idsex{$aim});
						#print "sex:$idsex{$aim}\n";
					}else{
						push (@pathways,0);
					}
					$loop_number++;
				}$j++;
			}$i++;
		}
	}
	return(\@pathways,$loop_number);
}

# this sub routine check if 2 arrays have same elements
# return 1 if true
# return 0 if faulse
# format: if_2_arys_have_same_elements(\@ary1,\@ary2);
sub if_2_arys_have_same_elements{
	my @ary1=@{$_[0]};
	my @ary2=@{$_[1]};
	my @temp1=();
	my @temp2=();
	my $return_value=undef;

	@temp1=(@ary1,@ary2);
	@temp2 = keys %{{map{$_,1} @temp1}};
	if ($#temp1==$#temp2){
		$return_value=0;
	} else { $return_value=1;}
	return ($return_value);
}



# this sub read pedigree data file and return familie ids
# the file should be space or comma dilimited format
sub Families{
	my $input_file=$_[0];
	open(INF,"$input_file") or
 	die "cannot open pedigree file $input_file\n$!\n";
	my @arr;
	my @p;
	while (<INF>) {
		chomp;
		next if (/^ *$/);
		split s/^ +//g;
		@arr = split /\s+/;
		@arr = split (/,/,$arr[0]) if ($#arr==0);
		push (@p,$arr[0]);
	}
	shift (@p)if ($p[0]==undef);
	@p= sort{$a<=>$b;} keys %{{map{$_,1}@p}};
	shift (@p)if ($p[0]==undef);
	pop (@p)if ($p[$#p]==undef);
	close (INF);
	return(@p);
}



# this sub read pedigree data file and list file
# extract information for 1 family each time
sub read_pedigree_data{
	my $input_pedigree_file=$_[0];
	my $current_family=$_[1];
	my $input_list_file=$_[2];
	my %idfa=undef;
	my %idmo=undef;
	my %idsex=undef;
	my %idfamily=undef;
	my @family=undef;
	my @id=undef;
	my @fa=undef;
	my @mo=undef;
	my @sex=undef;
	my @phenotyped_id=undef;

	my $i=0;
	my @arr=();
	open(INF,"$input_pedigree_file") or
 		die "cannot open pedigree file $input_pedigree_file\n$!\n";
	while (<INF>) {
		chomp;
		next if (/^$/ or /^ +$/ or /^#/);
		split s/^ +//g;
		@arr = split /\s+/;
		@arr = split (/,/,$arr[0]) if ($#arr==0);
		if ($arr[0]==$current_family){
			$family[$i]=$arr[0];
			$id[$i]=$arr[1];
			$fa[$i]=$arr[2];
			$mo[$i]=$arr[3];
			$sex[$i]=$arr[4];
			$i++;
		}
	}
	@arr=();
	close (INF);
	
	open (LIST, "$input_list_file") or die "cannot open list file $input_list_file\n$!\n";
	$i=0;
	while(<LIST>) {
		chomp;
		next if (/^$/ or /^ +$/ or /^#/);
		split s/^ +//g;
		@arr=split /\s+/;
		@arr=split(/,/,$arr[0]) if ($#arr==0);
		if ($arr[0]==$current_family){
			push (@phenotyped_id,$arr[1]);
		}
	}
	close (LIST);

	@idfa{@id}=@fa;
	@idmo{@id}=@mo;
	@idsex{@id}=@sex;
	@idfamily{@id}=@family;
	shift (@phenotyped_id) if ($phenotyped_id[0]==undef);
	return (\@id,\%idfa,\%idmo,\%idsex,\%idfamily,\@phenotyped_id);
}



# this sub routine read inter file
# each value is comma delimited
# returns a matrix,it's max line No. and it's max column No.
sub rd {
	my $in=$_[0];
	open (IN,"$in") or die "$!\n";
	my $i=0;
	my $j=0;
	my $value;
	my @arr;
	my $line;
	my $colum;
	my @length;
	while (<IN>){
		chomp;
		split s/^\s+//g;
		@arr=split /,/;
		if (@arr!=undef){
			push (@length,$#arr);
			for $j(0..$#arr){
				$value->[$i]->[$j]=$arr[$j];
			}
			$i++;
		}
	}
	close (IN);
	$line=$i-1;
	@length=sort {$a<=>$b} (@length);
	$colum=$length[$#length];

	# return a matix of whole data base
	# and index of lines and colum
	return ($value,$line,$colum);
}



# this sub routine calculate inbreeding coefficient
# format: $return=inbrdcf(@pathways);
sub inbrdcf{
	my @meiosis=@_;
	my @meiosis2=();
	my $inbrdcf=0;
	splice (@meiosis,0,4);
	foreach ($j=0;$j<=$#meiosis;$j++){
		if ($j%3==0){
			push 
				(@meiosis2,$meiosis[$j+1]+$meiosis[$j+2]);
		}
	}
	if ($meiosis2[0]==undef){
		$inbrdcf=0;
	}else{  
		$inbrdcf=0;
		foreach (@meiosis2){
			$inbrdcf+=0.5**$_*2;
		}
	}
	return ($inbrdcf);
}
	


# this sub routine calculate kinship coefficient
# format: $return=knshp(@pathways);
sub knshp{
	my @meiosis=@_;
	my @meiosis2=();
	my $knshpcf=0;
	splice (@meiosis,0,4);
	foreach ($j=0;$j<=$#meiosis;$j++){
		if ($j%5==0){
			push 
				(@meiosis2,$meiosis[$j+1]+$meiosis[$j+3]);
		}
	}if ($meiosis2[0]!=0){  
		$knshpcf=0;
		foreach (@meiosis2){
			$knshpcf+=0.5**$_*2;
		}
	}
	$knshpcf=$knshpcf/4;
	return ($knshpcf);
}

# this sub return the smallest number of an array of numbers
# $smallest = smallest(\@array)
#
sub smallest{
	my $aryref= shift;
	@$aryref=sort{$a<=>$b}(@$aryref);
	my $smallest= shift (@$aryref);
	if ($smallest){
		return ($smallest);
	}else{
		return (0);
	}
}

# this sub return the biggest number of an array of numbers
# $biggest = beggest (\@array)
#
sub biggest {
	my $aryref=shift;
	@$aryref=sort{$a<=>$b}(@$aryref);
	my $biggest = pop (@$aryref);
	if ($biggest){
		return ($biggest);
	} else {return (0)};
}

# this sub computes the odd median of an array of numbers.
# $median = median(\@array) 

sub median { 
	my $median = $_[0]->[ @{$_[0]} / 2 ] ;
	if ($median){
		return ($median); 
	} else { 
		return (0);
	}
}


# this sub compute standard deviation of an array of numbers
# name:		standard_deviation
# usage: 	$sd = standard_deviation(\@array); 
# require:	sub_routine mean.

sub standard_deviation {
    my $arrayref = shift;
    my $mean = mean($arrayref);
    if ($mean != 0){
	return sqrt( mean( [map $_ ** 2, @$arrayref] ) - ($mean ** 2) );
    } else {
	return (0);
    }
}

# this sub compute the mean of an array of numbers
# $mean = mean(\@array) 
#
sub mean {
    my ($arrayref) = @_;
    my $sum;
    foreach (@$arrayref) { $sum += $_ }
    if (@$arrayref != 0){
	    return $sum / @$arrayref;
    } else { return (0);}
}


# name:	help
# usage: &help($topic);
# $topic = 0 or undefined if no need help
#	-1 if general help
# 	others for topic help
sub help{
#	my $help = $_[0];
	while ($help!=0){
	if ($help ==1){
		print "
INSTALLATION

This program is written in perl, which can be run under Windows or Unix (Linux) operation system. If you are using Unix 
(Linux) operation system, you can download the file fcn.tar.gz at http://www.geneticepi.com/. After downloading the 
file, put it to a user-defined folder and type the command tar -xzvf fcn.tar.gz to unzip it. Windows users can unzip the 
file with WinZip but might need to install the platform of perl to run the program, which can be 
downloaded for free at http://www.activestate.com/Products/ActivePerl/.

After unzipping the file, you will find a new folder 'fcn', which consists of 2 files (readme.txt, fcn.pl) and 2 folders 
(doc, example). The file 'readme.txt' records the general information of all files in each folder and a short 
instruction on how to run the program. The file 'fcn.pl' is the executable program. In the folder 'doc', you can find 
the software manual, 'fcn_manual.txt', the documentation of this program 'fcn.doc' and GUN general public lincense. In 
another folder 'example', you can find a file, which records the example command, an example input pedigree file, and an 
example input list file.

"
	}elsif ($help == 2){
		print "
USAGE: 	perl fcn.pl OPTIONS
OPTIONS might be empty, and then the program asks one by one for obligatory options and tries default optional options.

	Options
	-p (pedigree):		the input pedigree file name
	-l (list):		the input list file name
	-so (standard-out): 	the standard output file name 
	-co (connection-out): 	the connection output file name <optional>
	-mp (ma/paternal):	the maternal/paternal specific connection output file name <optional>
	-ik (inbr/kin): 	inbreeding/kinship out put file name <optional>
	-io (inbreeding only):	if you want deal with only consanguineous loops (0 or 1) <optional>
	-h (help):		for more information <optional>

	See OPTIONS for more details on -so, -co, -mp, -ik, -io and -h options.

";
	}elsif ($help == 3){
		print "
OPTIONS

OPTION -so (string)
Specify the standard output file name. This output file gives a descriptive statistics of the distribution of the 
loops/connections. (Add \".csv\" extension if you want the file to be one-click open by Excel or StarCalc)
	
OPTION -co (string)
Specify the connection output file name. This output file lists out all consanguineous loops for each individual of 
interest and all consanguineous connections between each pair of individuals of interest.

OPTION -mp (string)
Specify the paternal/maternal specific connection output file name. This file lists all loops/connections in a 
paternal/maternal specific way. (This file might be very large depending on the pedigree size, pedigree structure, and 
number of interest people in the list file).
 
OPTION -ik (string)
Specify the inbreeding/kinship coefficient output file name. This file gives inbreeding coefficients taking into account 
only the shortest, mean, or the longest loops/connections as well as general inbreeding/kinship coefficients. 

OPTION -io (0 or 1)
If you are only interested in finding consanguineous loops and inbreeding values but not pair-wise connections and 
kinship values, you can specify 1 here, then the program runs much faster for large pedigrees. The default value of this 
option is 0.

OPTION -help (integer 0-9 or omit)
Help displays help information for fcn commands and features. Typing -help, -h or -? in the command line produces a list 
of following topics.
1 INSTALLATION
2 USAGE
3 OPTIONS
4 OUTPUT TO THE SCREEN
5 OUTPUT FILE FORMAT
6 EXAMPLE
7 REFERENCE
8 CONTACT THE AUTHORS
9 LICENSE
For help on a specific topic type -help <topic number>, for example, the command 
perl fcn.pl -h 2 will give help on the usage of the program. If you omit the topic number in the command line, the 
program will ask you to choose one.
The help text is an exact duplicate of the software manual which accompanies the distribution (in the directory \"doc\").
";
	}elsif ($help == 4){
		print "
INPUT FILE FORMAT

You need 2 input files to run this program: a pedigree file and a list file. Both these 2 input files should be in 
space, tab or comma delimited format.  To run the program, you need to put these 2 input files and the executable file 
(fcn.pl) into the same directory, and then type the command line in DOS or UNIX (Linux) prompt. And if you need to 
transform files from windows to Unix/Linux, please remember dos2unix these input files.

Pedigree File (-p)
The pedigree file consists of genealogy information in a standard LINKAGE pedigree file format.
The first 5 columns should consist of following information:
Column1: family identification (ID)
Column2: individual ID
Column3: father's ID
Column4: mother's ID
Column5: sex (1 man, 2 women)
Please note that for each family, individual's ID should be unique. Column3 and column4 are both zero when the 
individual described in column2 is a founder. Any column after the 5th column is ignored.

List File (-l)
The list file consists a group of individual of interest (normally those who have phenotype and genotype information).
The format of list file:
Column1: family ID
Column2: individual ID

"; 
	}elsif($help==5){
		print "

OUTPUT FILE FORMAT
	
The standard output file (-so)
	Column1: family ID
	Column2: ID1
	Column3: ID2
	Column4: total number of loops/connections
	Column5: number of meiosis of the shortest loop/connection
	Column6: number of meiosis of the longest loop/connection
	Column7: median of loop/connection length
	Column8: mean of loop/connection length
	Column9: standard deviation of the loop/connection length

The connection output file (-co)
	Column1: family ID
	Column2: the first individual (ID1)
	Column3: the second individual (ID2) 
Column4: 
total number of consanguineous loops (when ID1 and ID2 are the same)
total number of consanguineous connections (when ID1 and ID2 are different)
	Column (5+3n): individual ID to whom a disjoint loop/connection exists
	Column (6+3n): number of meiosis of a paternal or maternal pathway to that ID
Column (7+3n): number of meiosis of the corresponding maternal or paternal pathway to that ID
	n = (0,1,2,...,n)

The inbreeding/kinship coefficient output file (-ik)
	Column1: family ID
	Column2: ID1
	Column3: ID2
	Column4: inbreeding coefficient (when ID and ID2 are the same)
 		     kinship coefficient (when ID1 and ID2 are different)

The paternal/maternal specific loop/connection output file (-mp)
	Column1: family ID
	Column2: ID1
	Column3: ID2
	Column4: total number of loops/connections
	Column (5+5n): individual ID to whom a disjoint loop/connection exists
	Column (6+5n): number of meioses of one pathway to the ID in column (5+5n)
	Column (7+5n): describe if this pathway is paternal or maternal (p = paternal, m = maternal, x = neither)
	Column (8+5n): number of meioses of another pathway to the ID in column (5+5n)
	Column (9+5n): describe if this pathway is paternal or maternal (p = paternal, m = maternal, x = neither)
	n = (0,1,2,...,n)

";
	} elsif ($help==6){
		print "
EXAMPLE

In the folder example, the file example_command.txt records the command to run the example. You can also find an 
example input pedigree file and an example input list file. 

To run the example, from the example directory type: 
perl fcn.pl -p example_ped.txt -l example_list.txt -so std.out -co connection.out -mp paternal_maternal.out -ik 
inbreeding_kinship.out 

If you miss at least one of the obligatory options (pedigree file, list file, and the standard output file), the program 
activates an interactive mode and asks for all the missing options one by one (including the optional options).

";
 	}elsif ($help == 7){
		print "
REFERENCE
	
	When using this software, please put reference to 
	F.Liu, S.Elefante, C.M. van Duijn, Y.S.Aulchenko.
	FCN(Finding connections): A program for characterization of complex pedigrees 

";	
	}elsif ($help ==8){
		print "
Thanks for your suggestions,comments or questions!
Please send an email to either of us:
F. Liu, Erasmus Medical Center Rotterdam 
f.liu\@erasmusmc.nl 
or
Y. Aulchenko, Erasmus Medical Center Rotterdam, Inst. Cytol. Genet. SD RAS 
i.aoultchenko\@erasmusmc.nl
";
	}elsif ($help == 9){
		print "
LICENSE

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public 
License as published by the Free Software Foundation; either version 1.3 of the License, or (at your option) any later 
version.
     
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY, without even the implied 
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
     
You should have received a copy of the GNU General Public License along with this program; if not, write to the Free 
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
";
	}
print "
Choose a topic for help (number 0-8):
0 done with help or the help does not really help (as I expected :-> )
1 INSTALLATION
2 USAGE
3 OPTIONS
4 OUTPUT TO THE SCREEN
5 OUTPUT FILE FORMAT
6 EXAMPLE
7 REFERENCE
8 CONTACT THE AUTHORS
9 LICENSE
Your choice is:
";
chomp($help=<STDIN>);
help($help);
}
}
 

# activate the interactive mode
sub interactive_mode{
	my $question=$_[0];
	if ($question==1){
		print "please specify the input pedigree file name: ";
		chomp($pedigree_file=<STDIN>);
		print "the pedigree file name is $pedigree_file\n";
	}elsif ($question==2){
		print "please specify the input list file name: ";
		chomp($list_file=<STDIN>);
		print "list file name is: $list_file\n";
	}elsif ($question==3){
		print "please specify the standard output file name: ";
		chomp($standard_output_file=<STDIN>);
		print "output file name now is: $standard_output_file\n";
	}
}


###	 END OF SUBS	 ### ###########################

